import xlrd





def get_excel_data(fireDir,sheetName):
    wookBook = xlrd.open_workbook(fireDir,formatting_info=True)
    sheet = wookBook.sheet_by_name(sheetName)
    project_name = sheet.cell_value(0,1)
    keys = sheet.col_values(3)[1:]
    value = sheet.col_values(4)[1:]
    step = sheet.col_values(1)[1:]
    step_text = sheet.col_values(2)[1:]
    reslist = []
    alist = []
    for one in range(len(keys)):
        if one == 0:
            continue
        reslist.append({keys[one]:value[one],step[one]:step_text[one]})
        if keys[one] == 'assert':
            alist.append(reslist)
            reslist = []
            continue
    input_list = [index for index,key in enumerate(keys) if key == 'input']
    click_list = [index for index,key in enumerate(keys) if key == 'click']
    move_on_list =  [index for index,key in enumerate(keys) if key == 'move_on']
    save_text_list = [index for index,key in enumerate(keys) if key == 'save_text']
    save_texts_list = [index for index,key in enumerate(keys) if key == 'save_texts']
    assert_list = [index for index,key in enumerate(keys) if key == 'assert']
    input_idx = 0
    click_idx = 0
    move_on_idx = 0
    save_text_idx = 0
    save_texts_idx = 0
    assert_idx = 0
    for datas in alist:
        for one in datas:
            hand_key = list(one.keys())[0]
            if hand_key == 'input':
                one[hand_key] = [sheet.cell_value(input_list[input_idx]+1 , 4),
                                 sheet.cell_value(input_list[input_idx]+1, 5),
                                 sheet.cell_value(input_list[input_idx]+1, 6)]
                input_idx+=1
            if hand_key == 'click':

                one[hand_key] = [sheet.cell_value(click_list[click_idx]+1 , 4),
                                 sheet.cell_value(click_list[click_idx]+1, 5)]
                click_idx+=1
            if hand_key == 'move_on':
                one[hand_key] = [sheet.cell_value(move_on_list[move_on_idx] + 1, 4),
                                 sheet.cell_value(move_on_list[move_on_idx] + 1, 5)]
                move_on_idx += 1

            if hand_key == 'assert':
                one[hand_key] = [sheet.cell_value(assert_list[assert_idx] + 1, 4),
                                 sheet.cell_value(assert_list[assert_idx] + 1, 5)]
                assert_idx += 1
            if hand_key == 'save_text':
                one[hand_key] = [sheet.cell_value(save_text_list[save_text_idx]+1 , 4),
                                 sheet.cell_value(save_text_list[save_text_idx]+1, 5),
                                 sheet.cell_value(save_text_list[save_text_idx]+1, 6)]
                save_text_idx+=1
            if hand_key == 'save_texts':
                one[hand_key] = [sheet.cell_value(save_texts_list[save_texts_idx]+1 , 4),
                                 sheet.cell_value(save_texts_list[save_texts_idx]+1, 5),
                                 sheet.cell_value(save_texts_list[save_texts_idx]+1, 6)]
                save_texts_idx+=1
    return alist,project_name



if __name__ == '__main__':
    print(get_excel_data('D:\教学资料\代码\pyrf\测试用例.xls','登录模块'))





import sys
import allure
import pytest
from .exec import Exec
from .utils.handle_excel import get_excel_data




filepath = sys.argv[1]
filesheet = sys.argv[2]

all_data = get_excel_data(filepath,filesheet)


@pytest.mark.parametrize('data',all_data[0])
@allure.epic(f'{all_data[1]}')
@allure.feature(f'{filesheet}')
@allure.title('{data[1][name]}')
def test_tem(data):
        exe = Exec(data)
        list_dict = {
            'goto':exe.goto,
            'input':exe.input,
            'click':exe.click,
            'assert':exe.def_assert,
            'save_text':exe.save_text
        }
        for one in data:
            for k  in list(one.keys()):
                if k in list_dict:
                    if k == 'save_text':
                        res = list_dict[k]()
                    elif k == 'assert':
                        list_dict[k](res)
                    else:
                        list_dict[k]()
                else:
                    continue





import pytest
import os



def test_run():
    pytest.main([os.path.join(os.path.dirname(os.path.abspath(__file__)),'test_template.py'),'-sv', '--alluredir', 'report/tmp'])
    os.system('allure serve report/tmp')




from selenium import webdriver
import allure
from .utils.handle_loguru import log
from time import strftime


class Exec:
    def __init__(self,data):
        self.data = data
        if 'browser' in self.data[0]:
            if self.data[0]['browser'] == 'chrome':
                self.driver = webdriver.Chrome()
            elif self.data[0]['browser'] == 'firefox':
                self.driver = webdriver.Firefox()
        self.driver.implicitly_wait(10)


    def goto(self):
        for one in self.data:
            if one.get('goto'):
                goto_data = one.pop('goto')
                step,step_text = list(one.items())[0]
                with allure.step(f'步骤{step}:{step_text}'):
                    self.driver.get(goto_data)
                    log.info(f'步骤{step}:{step_text}')
                    file_name = f'{strftime("%Y%m%d%H%M%S")}{step_text}.png'
                    self.driver.save_screenshot(file_name)
                    with open(file_name,'rb') as f:
                        file = f.read()
                    allure.attach(file,attachment_type=allure.attachment_type.PNG)
                    break
        return self

    def input(self):
        for one in self.data:
            if one.get('input'):
                input_ele = one.pop('input')
                step,step_text = list(one.items())[0]
                with allure.step(f'步骤{step}:{step_text}'):
                    self.driver.find_element(*input_ele[:-1]).send_keys(input_ele[-1])
                    log.info(f'步骤{step}:{step_text}')
                    log.info(f'定位方式by={input_ele[:-1][0]},定位的值{input_ele[:-1][-1]}')
                    log.info(f'输入的值{input_ele[-1]}')
                    file_name = f'{strftime("%Y%m%d%H%M%S")}{step_text}.png'
                    self.driver.save_screenshot(file_name)
                    with open(file_name, 'rb') as f:
                        file = f.read()
                    allure.attach(file, attachment_type=allure.attachment_type.PNG)
                    break
        return self

    def click(self):
        for one in self.data:
            if one.get('click'):
                click_data = one.pop('click')
                step, step_text = list(one.items())[0]
                with allure.step(f'步骤{step}:{step_text}'):
                    self.driver.find_element(*click_data).click()
                    log.info(f'步骤{step}:{step_text}')
                    log.info(f'定位方式by={click_data[0]},定位的值{click_data[-1]}')
                    log.info('点击元素')
                    file_name = f'{strftime("%Y%m%d%H%M%S")}{step_text}.png'
                    self.driver.save_screenshot(file_name)
                    with open(file_name, 'rb') as f:
                        file = f.read()
                    allure.attach(file, attachment_type=allure.attachment_type.PNG)
                    break
        return self


    def def_assert(self,save_data):
        var = save_data[0]
        result = save_data[1]
        for one in self.data:
            if one.get('assert'):
                assert_data = one.pop('assert')
                step, step_text = list(one.items())[0]
                with allure.step(f'步骤{step}:{step_text}'):
                    if var in assert_data[-1]:
                        assert_data[-1] = result
                        try:
                            if isinstance(result,list):
                                log.info(f'步骤{step}:{step_text}')
                                log.info(f'期望值{assert_data[0]},实际值{result}')
                                assert assert_data[0] in result
                            else:
                                log.info(f'步骤{step}:{step_text}')
                                log.info(f'期望值{assert_data[0]},实际值{result}')
                                assert assert_data[0] == result
                        except Exception as error:
                            log.info(f'步骤{step}:{step_text}')
                            log.error(f'断言失败，期望值{assert_data[0]},实际值{result}')
                            file_name = f'{strftime("%Y%m%d%H%M%S")}{step_text}断言失败.png'
                            self.driver.save_screenshot(file_name)
                            with open(file_name, 'rb') as f:
                                file = f.read()
                            allure.attach(file, attachment_type=allure.attachment_type.PNG)
                            raise error
                    else:
                        log.info(f'没有找到变量{assert_data[-1]}')
                        raise Exception('变量填写错误')



    def save_text(self):
        for one in self.data:
            if one.get('save_text'):
                text_ele = one.pop('save_text')
                step, step_text = list(one.items())[0]
                with allure.step(f'步骤{step}:{step_text}'):
                    text_msg = self.driver.find_element(*text_ele[:-1]).text
                    var = text_ele[-1]
                    log.info(f'步骤{step}:{step_text}')
                    log.info(f'保存变量{var},变量值为{text_msg}')
                    file_name = f'{strftime("%Y%m%d%H%M%S")}{step_text}.png'
                    self.driver.save_screenshot(file_name)
                    with open(file_name, 'rb') as f:
                        file = f.read()
                    allure.attach(file, attachment_type=allure.attachment_type.PNG)
        return var,text_msg


    def save_texts(self):
        for one in self.data:
            if one.get('save_texts'):
                text_ele = one.pop('save_texts')
                step, step_text = list(one.items())[0]
                with allure.step(f'步骤{step}:{step_text}'):
                    text_msg = [ele.text for ele in self.driver.find_elements(*text_ele[:-1])]
                    var = text_ele[-1]
                    log.info(f'步骤{step}:{step_text}')
                    log.info(f'保存变量{var},变量值为{text_msg}')
                    file_name = f'{strftime("%Y%m%d%H%M%S")}{step_text}.png'
                    self.driver.save_screenshot(file_name)
                    with open(file_name, 'rb') as f:
                        file = f.read()
                    allure.attach(file, attachment_type=allure.attachment_type.PNG)
        return var, text_msg



from loguru import logger
import logging

from time import strftime
import os


class PropogateHandler(logging.Handler):
    def emit(self, record):
        logging.getLogger(record.name).handle(record)



class MyLog():
    __instance = None  # 单例实现
    __call_flag = True  # 控制init调用，如果调用过就不再调用

    def __new__(cls, *args, **kwargs):
        if not cls.__instance:
            cls.__instance = super().__new__(cls)
        return cls.__instance

    # def __init__(self):
    def get_log(self):
        if self.__call_flag:  # 看是否调用过
            __curdate = strftime('%Y%m%d-%H%M%S')
            logger.remove(handler_id=None)  # 关闭console输出
            logger.add('log/' + __curdate + '.log', encoding='utf8', # 日志存放位置
                       retention='2 days',  # 清理
                       rotation='10 MB',  # 循环 达到指定大小后建立新的日志
                       format='{time:YYYY-MM-DD HH:mm:ss},{module}(line:{line}),{level}|{message}',  # 日志输出格式
                       compression='zip',  # 日志压缩格式
                       level='INFO',
                       )  # 日志级别
            logger.add(PropogateHandler())
            self.__call_flag = False  # 如果调用过就置为False
        return logger

log = MyLog().get_log()

if __name__ == '__main__':
    print(os.path.dirname(__file__))

